@@ -1,9 +1,9 @@
 /*
   zip_stat_index.c -- get information about file by index
-  Copyright (C) 1999-2020 Dieter Baron and Thomas Klausner
+  Copyright (C) 1999-2024 Dieter Baron and Thomas Klausner
 
   This file is part of libzip, a library to manipulate ZIP archives.
-  The authors can be contacted at <libzip@nih.at>
+  The authors can be contacted at <info@libzip.org>
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -39,24 +39,45 @@ ZIP_EXTERN int
 zip_stat_index(zip_t *za, zip_uint64_t index, zip_flags_t flags, zip_stat_t *st) {
     const char *name;
     zip_dirent_t *de;
+    zip_entry_t *entry;
 
-    if ((de = _zip_get_dirent(za, index, flags, NULL)) == NULL)
+    if ((de = _zip_get_dirent(za, index, flags, NULL)) == NULL) {
         return -1;
+    }
 
-    if ((name = zip_get_name(za, index, flags)) == NULL)
+    if ((name = zip_get_name(za, index, flags)) == NULL) {
         return -1;
+    }
 
+    entry = za->entry + index;
 
     if ((flags & ZIP_FL_UNCHANGED) == 0 && ZIP_ENTRY_DATA_CHANGED(za->entry + index)) {
-        zip_entry_t *entry = za->entry + index;
 
         if (zip_source_stat(entry->source, st) < 0) {
             zip_error_set(&za->error, ZIP_ER_CHANGED, 0);
             return -1;
         }
 
+        if (ZIP_CM_IS_DEFAULT(de->comp_method)) {
+            if (!(st->valid & ZIP_STAT_COMP_METHOD) || st->comp_method == ZIP_CM_STORE) {
+                st->valid &= ~(ZIP_STAT_COMP_SIZE|ZIP_STAT_COMP_METHOD);
+            }
+        }
+        else {
+            if ((st->valid & ZIP_STAT_COMP_METHOD) && st->comp_method != de->comp_method) {
+                st->valid &= ~ZIP_STAT_COMP_SIZE;
+            }
+            st->valid |= ZIP_STAT_COMP_METHOD;
+            st->comp_method = de->comp_method;
+        }
+
+        if (((st->valid & (ZIP_STAT_COMP_METHOD|ZIP_STAT_SIZE)) == (ZIP_STAT_COMP_METHOD|ZIP_STAT_SIZE)) && st->comp_method == ZIP_CM_STORE) {
+            st->valid |= ZIP_STAT_COMP_SIZE;
+            st->comp_size = st->size;
+        }
+
         if (entry->changes != NULL && entry->changes->changed & ZIP_DIRENT_LAST_MOD) {
-            st->mtime = de->last_mod;
+            st->mtime = _zip_d2u_time(&de->last_mod);
             st->valid |= ZIP_STAT_MTIME;
         }
     }
@@ -65,11 +86,22 @@ zip_stat_index(zip_t *za, zip_uint64_t index, zip_flags_t flags, zip_stat_t *st)
 
         st->crc = de->crc;
         st->size = de->uncomp_size;
-        st->mtime = de->last_mod;
+        st->mtime = _zip_d2u_time(&de->last_mod);
         st->comp_size = de->comp_size;
         st->comp_method = (zip_uint16_t)de->comp_method;
         st->encryption_method = de->encryption_method;
         st->valid = (de->crc_valid ? ZIP_STAT_CRC : 0) | ZIP_STAT_SIZE | ZIP_STAT_MTIME | ZIP_STAT_COMP_SIZE | ZIP_STAT_COMP_METHOD | ZIP_STAT_ENCRYPTION_METHOD;
+        if (entry->changes != NULL && entry->changes->changed & ZIP_DIRENT_COMP_METHOD) {
+            st->valid &= ~ZIP_STAT_COMP_SIZE;
+        }
+    }
+
+    if ((za->ch_flags & ZIP_AFL_WANT_TORRENTZIP) && (flags & ZIP_FL_UNCHANGED) == 0) {
+        zip_dostime_t dostime = {0xbc00, 0x2198};
+        st->comp_method = ZIP_CM_DEFLATE;
+        st->mtime = _zip_d2u_time(&dostime);
+        st->valid |= ZIP_STAT_MTIME | ZIP_STAT_COMP_METHOD;
+        st->valid &= ~ZIP_STAT_COMP_SIZE;
     }
 
     st->index = index;
